# Efficient and Robust Estimation of the Generliazed LATE Model
# Empirical Application: Oregon Health Insurance Experiment
# Haitian Xie

# The data is downloaded from Oregon health insurance experiment public use data. Available at
# http://www.nber.org/oregon/data.html

library(haven)
library(dplyr)
library(glmnet)

#################
# Load the data #
#################

descriptive <- read_dta("oregonhie_descriptive_vars.dta")
ed <- read_dta("oregonhie_ed_vars.dta")
survey12m <- read_dta("oregonhie_survey12m_vars.dta")

data <- inner_join(descriptive,ed,by = "person_id") %>% inner_join(survey12m, by = "person_id")
rm(descriptive,ed,survey12m)

# variable names
outcome_var <- c("doc_num_mod_12m","cost_tot_oop_12m","baddays_tot_12m")
treatment_var <- c("ins_noins_12m","ins_ohp_12m","ins_private_12m","ins_othcov_12m",
                   "ins_employer_12m","ins_medicare_12m")
instrument_var <- c("treatment")
covariate_var <- c("numhh_list","wave_survey12m","birthyear_list","have_phone_list","english_list","female_list",
                   "first_day_list","last_day_list","pobox_list","self_list","week_list","num_visit_pre_cens_ed")

data <- data[,c(outcome_var,treatment_var,instrument_var,covariate_var)]
data <- na.omit(data)

########################
# Define the variables #
########################

# covariates/controls

X <- data[,covariate_var]
X$numhh_list <- as.factor(X$numhh_list) # factorize household size
X$wave_survey12m <- as.factor(X$wave_survey12m) # factorize survey wave
X$age <- cut(X$birthyear_list,breaks = c(1945,1960,1970,1980,1990),labels=1:4,include.lowest = TRUE) 
X <- subset(X, select = -birthyear_list)
X <- model.matrix(~.,X)


# instrument
Z <- data$treatment

# treatment
Treatment <- vector(length = nrow(data))
T1 <- data$ins_noins_12m == 1
T2 <- data$ins_private_12m == 1 | data$ins_othcov_12m == 1 | data$ins_employer_12m == 1 | data$ins_medicare_12m == 1 
T3 <- data$ins_ohp_12m == 1
Treatment[T1] <- "no"
Treatment[T2] <- "nm"
Treatment[T3] <- "m"
Treatment <- as.factor(Treatment)

# outcome

Y_doctor_visit <- data$doc_num_mod_12m
Y_financial <- pmin(data$cost_tot_oop_12m,quantile(data$cost_tot_oop_12m,probs = 0.98))
Y_self_report <- data$baddays_tot_12m


##############
# Estimation #
##############

n <- nrow(data) # sample size
k <- 2 # number of folds

# create matrices that store \hat{\eta}(X_i), 
# where \hat{\eta} = \hat{P}_{t,z} or \hat{Q}_{t,z} or \hat{\pi}_{z} is fitted 
# on the k-1 folds that do not contain observation i

no.never <- matrix(nrow = floor(n/k), ncol = k)
nm.never <- matrix(nrow = floor(n/k), ncol = k)
always <- matrix(nrow = floor(n/k), ncol = k)
no.complier <- matrix(nrow = floor(n/k), ncol = k)
nm.complier <- matrix(nrow = floor(n/k), ncol = k)
complier <- matrix(nrow = floor(n/k), ncol = k)

beta12_num <- matrix(nrow = floor(n/k), ncol = k)
beta22_num <- matrix(nrow = floor(n/k), ncol = k)
beta32_num <- matrix(nrow = floor(n/k), ncol = k)
beta11_num <- matrix(nrow = floor(n/k), ncol = k)
beta21_num <- matrix(nrow = floor(n/k), ncol = k)
beta31_num <- matrix(nrow = floor(n/k), ncol = k)

#cross-fitting

set.seed(123)
id <- sample(rep(1:k,floor(n/k)))

for (fold in 1:k){
  
  # define training and test sets 
  
  train_index <- id != fold
  test_index <- id == fold
  
  
  #Y.train <- Y_doctor_visit[train_index] # (switch between different outcome variables)
  #Y.train <- Y_financial[train_index]
  Y.train <- Y_self_report[train_index] 
  Treatment.train <- Treatment[train_index]
  Z.train <- Z[train_index]
  X.train <- X[train_index,]
  
  #Y.test <- Y_doctor_visit[test_index] # (switch between different outcome variables)
  #Y.test <- Y_financial[test_index]
  Y.test <- Y_self_report[test_index]
  Treatment.test <- Treatment[test_index]
  Z.test <- Z[test_index]
  X.test <- X[test_index,]
  
  # estimate instrument propensity \pi by logistic lasso 
  
  cv.pi <- cv.glmnet(X.train,Z.train,family = "binomial",alpha=1)
  pi <- glmnet(X.train,Z.train,family = "binomial",alpha=1,lambda = cv.pi$lambda.min)
  
  # estimate P_{t,z} by multinomial logistic lasso (six of them)
  
  cv.Pt0 <- cv.glmnet(X.train[Z.train==0,],Treatment.train[Z.train==0],family = "multinomial",alpha=1)
  Pt0 <- glmnet(X.train[Z.train==0,],Treatment.train[Z.train==0],family = "multinomial",alpha=1,lambda=cv.Pt0$lambda.min)
  
  cv.Pt1 <- cv.glmnet(X.train[Z.train==1,],Treatment.train[Z.train==1],family = "multinomial",alpha=1)
  Pt1 <- glmnet(X.train[Z.train==1,],Treatment.train[Z.train==1],family = "multinomial",alpha=1,lambda=cv.Pt1$lambda.min)
  
  # estimate Q_{t,z} by lasso (six of them)
  
  cv.Q10 <- cv.glmnet(X.train[Z.train==0,],(Y.train*(Treatment.train=="no"))[Z.train==0],family = "gaussian",alpha=1)
  Q10 <- glmnet(X.train[Z.train==0,],(Y.train*(Treatment.train=="no"))[Z.train==0],family = "gaussian",alpha=1,lambda=cv.Q10$lambda.min)
  
  cv.Q11 <- cv.glmnet(X.train[Z.train==1,],(Y.train*(Treatment.train=="no"))[Z.train==1],family = "gaussian",alpha=1)
  Q11 <- glmnet(X.train[Z.train==1,],(Y.train*(Treatment.train=="no"))[Z.train==1],family = "gaussian",alpha=1,lambda=cv.Q11$lambda.min)
  
  cv.Q20 <- cv.glmnet(X.train[Z.train==0,],(Y.train*(Treatment.train=="nm"))[Z.train==0],family = "gaussian",alpha=1)
  Q20 <- glmnet(X.train[Z.train==0,],(Y.train*(Treatment.train=="nm"))[Z.train==0],family = "gaussian",alpha=1,lambda=cv.Q20$lambda.min)
  
  cv.Q21 <- cv.glmnet(X.train[Z.train==1,],(Y.train*(Treatment.train=="nm"))[Z.train==1],family = "gaussian",alpha=1)
  Q21 <- glmnet(X.train[Z.train==1,],(Y.train*(Treatment.train=="nm"))[Z.train==1],family = "gaussian",alpha=1,lambda=cv.Q21$lambda.min)
  
  cv.Q30 <- cv.glmnet(X.train[Z.train==0,],(Y.train*(Treatment.train=="m"))[Z.train==0],family = "gaussian",alpha=1)
  Q30 <- glmnet(X.train[Z.train==0,],(Y.train*(Treatment.train=="m"))[Z.train==0],family = "gaussian",alpha=1,lambda=cv.Q30$lambda.min)
  
  cv.Q31 <- cv.glmnet(X.train[Z.train==1,],(Y.train*(Treatment.train=="m"))[Z.train==1],family = "gaussian",alpha=1)
  Q31 <- glmnet(X.train[Z.train==1,],(Y.train*(Treatment.train=="m"))[Z.train==1],family = "gaussian",alpha=1,lambda=cv.Q31$lambda.min)
  
  # evaluate the nonparametric estimators at each value in the test set
  for (i in 1:length(Y.test))
  {
    
    # construct the observations for case i
    Yi <- Y.test[i]
    T1i <- Treatment.test[i] == "no"
    T2i <- Treatment.test[i] == "nm"
    T3i <- Treatment.test[i] == "m"
    Zi <- Z.test[i]
    Xi <- X.test[i,,drop=FALSE]
    
    # evaluate pi, P_{t,z}, Q_{t,z} at X_i
    pii <- predict(pi,newx = Xi, type="response")
    
    P10i <- predict(Pt0,newx = Xi, type="response")[1]
    P20i <- predict(Pt0,newx = Xi, type="response")[2]
    P30i <- predict(Pt0,newx = Xi, type="response")[3]
    P11i <- predict(Pt1,newx = Xi, type="response")[1]
    P21i <- predict(Pt1,newx = Xi, type="response")[2]
    P31i <- predict(Pt1,newx = Xi, type="response")[3]
    
    Q10i <- predict(Q10,newx = Xi, type="link")
    Q11i <- predict(Q11,newx = Xi, type="link")
    Q20i <- predict(Q20,newx = Xi, type="link")
    Q21i <- predict(Q21,newx = Xi, type="link")
    Q30i <- predict(Q30,newx = Xi, type="link")
    Q31i <- predict(Q31,newx = Xi, type="link")
    
    # store those values in the matrices 
    no.never[i,fold] <- P11i + (Zi)/(pii) * (T1i - P11i)
    nm.never[i,fold] <- P21i + (Zi)/(pii) * (T2i - P21i)
    always[i,fold] <- P30i + (1-Zi)/(1-pii) * (T3i - P30i)
    no.complier[i,fold] <- P10i - P11i + (1-Zi)/(1-pii) * (T1i - P10i) - (Zi)/(pii) * (T1i - P11i)
    nm.complier[i,fold] <- P20i - P21i + (1-Zi)/(1-pii) * (T2i - P20i) - (Zi)/(pii) * (T2i - P21i)
    complier[i,fold] <- P31i - P30i + (Zi)/(pii) * (T3i - P31i) - (1-Zi)/(1-pii) * (T3i - P30i)
    
    beta12_num[i,fold] <- Q11i + (Zi)/(pii) * (Yi*T1i - Q11i)
    beta22_num[i,fold] <- Q21i + (Zi)/(pii) * (Yi*T2i - Q21i)
    beta32_num[i,fold] <- Q30i + (1-Zi)/(1-pii) * (Yi*T3i - Q30i)
    beta11_num[i,fold] <- Q10i - Q11i + (1-Zi)/(1-pii) * (Yi*T1i - Q10i) - (Zi)/(pii) * (Yi*T1i - Q11i)
    beta21_num[i,fold] <- Q20i - Q21i + (1-Zi)/(1-pii) * (Yi*T2i - Q20i) - (Zi)/(pii) * (Yi*T2i - Q21i)
    beta31_num[i,fold] <- Q31i - Q30i + (Zi)/(pii) * (Yi*T3i - Q31i) - (1-Zi)/(1-pii) * (Yi*T3i - Q30i)
  }
  
}

# compute p_{t,k} and standard errors

p12 <- round(mean(no.never),3)
sp12 <- round(sd(no.never)/sqrt(n),3)

p22 <- round(mean(nm.never),3)
sp22 <- round(sd(nm.never)/sqrt(n),3)

p32 <- round(mean(always),3)
sp32 <- round(sd(always)/sqrt(n),3)

p11 <- round(mean(no.complier),3)
sp11 <- round(sd(no.complier)/sqrt(n),3)

p21 <- round(mean(nm.complier),3)
sp21 <- round(sd(nm.complier)/sqrt(n),3)

p31 <- round(mean(complier),3)
sp31 <- round(sd(complier)/sqrt(n),3)

print(c(p12,p22,p32,p11,p21,p31))
print(c(sp12,sp22,sp32,sp11,sp21,sp31))

# compute \beta_{t,k} and standard errors

beta12 <- round(mean(beta12_num)/p12,2)
beta22 <- round(mean(beta22_num)/p22,2)
beta32 <- round(mean(beta32_num)/p32,2)
beta11 <- round(mean(beta11_num)/p11,2)
beta21 <- round(mean(beta21_num)/p21,2)
beta31 <- round(mean(beta31_num)/p31,2)

sigma12 <- round(sd(beta12_num - beta12 * no.never)/p12/sqrt(n),2)
sigma22 <- round(sd(beta22_num - beta22 * nm.never)/p22/sqrt(n),2)
sigma32 <- round(sd(beta32_num - beta32 * always)/p32/sqrt(n),2)
sigma11 <- round(sd(beta11_num - beta11 * no.complier)/p11/sqrt(n),2)
sigma21 <- round(sd(beta21_num - beta21 * nm.complier)/p21/sqrt(n),2)
sigma31 <- round(sd(beta31_num - beta31 * complier)/p31/sqrt(n),2)

print(c(beta12,beta22,beta32,beta11,beta21,beta31))
print(c(sigma12,sigma22,sigma32,sigma11,sigma21,sigma31))
